
package app;

import java.util.ArrayList;
import java.util.List;

/**
 * MathLogic
 */
public class MathLogic {

    /**
     * Get a list of divisors
     * 
     * @param input Integer from which you want to get all divisors
     * @return List of Divisors
     */
    private static List<Integer> getDivisors(int input) {
        List<Integer> divisors = new ArrayList<Integer>();
        for (int i = 1; i < input; i++) {
            if (input % i == 0) {
                divisors.add(i);
            }
        }
        return divisors;
    }

    /**
     * Sum all divisors
     * 
     * @param input number wich the divisors shall be summed
     * @return added up divisors
     */
    private static int sumDivisors(int input) {
        List<Integer> divisors = getDivisors(input);
        int count = 0;
        for (int divisor : divisors) {
            count += divisor;
        }
        return count;
    }

    /**
     * Test entered number for abundanz
     * 
     * @param input number wich shall be tested
     * @return if `input` is abundant or not?
     */
    public static boolean isAbundant(int input) {
        int count = sumDivisors(input);
        return count > input;
    }

    /**
     * Test entered number for perfect
     * 
     * @param input number which shall be tested
     * @return if `input` is perfect or not?
     */
    public static boolean isPerfect(int input) {
        int count = sumDivisors(input);
        return count == input;
    }

    /**
     * Test entered number for Harshad-Zahl
     * 
     * @param input number which shall be tested
     * @return if `input` is a Harshad-Zahl or not?
     */
    public static boolean isHarshad(int input) {
        int num = input;
        int sum = 0;
        while (num > 0) {
            sum = sum + num % 10;
            num = num / 10;
        }
        return (input % sum) == 0;
    }

    /**
     * Print everything
     * 
     * @param limit number from 1 where to go
     */
    public static void printAllLists(int limit) {
        List<Integer> abundants = new ArrayList<Integer>();
        List<Integer> perfects = new ArrayList<Integer>();
        List<Integer> harshads = new ArrayList<Integer>();
        for (int i = 1; i <= limit; i++) {
            if (isAbundant(i))
                abundants.add(i);
            if (isPerfect(i))
                perfects.add(i);
            if (isHarshad(i))
                harshads.add(i);
        }
        System.out.println("All abundant numbers from 1 to " + limit);
        System.out.println(abundants);
        System.out.println("All perfect numbers from 1 to " + limit);
        System.out.println(perfects);
        System.out.println("All harshad numbers from 1 to " + limit);
        System.out.println(harshads);
    }

}